@@ -9,7 +9,7 @@
 #![feature(drain_filter)]
 #![feature(result_cloned)]
 #![feature(specialization)]
-#![feature(min_type_alias_impl_trait)]
+#![feature(type_alias_impl_trait)]
 #![feature(associated_type_bounds)]
 #![feature(trait_alias)]
 #![feature(generic_associated_types)]
